#ifndef ATBDEVICECONTROLLERPLUGIN_H
#define ATBDEVICECONTROLLERPLUGIN_H

#include <QObject>

#include "src/ATBAPP/DeviceControllerInterface.h"
#include "src/ATBAPP/ATBAPPplugin.h"
#include "src/ATBAPP/DeviceControllerDiag.h"

#include "version.h"



#include "interfaces.h"


class DBusControllerInterface;
class QTextCodec;


// using namespace nsDeviceControllerInterface;

class QSettings;

class ATBDeviceControllerPlugin : public DeviceControllerInterface
{
    Q_OBJECT
    Q_INTERFACES(ATBAPPplugin)
    Q_INTERFACES(DeviceControllerInterface)
    #if QT_VERSION >= 0x050000
    Q_PLUGIN_METADATA( IID "ATBDeviceControllerPlugin" )
    #endif

public:
    explicit ATBDeviceControllerPlugin(QObject *parent = nullptr);
    virtual ~ATBDeviceControllerPlugin();

    // ----------------------------------------------------------------------------
    // interface:
    virtual PLUGIN_STATE initPlugin(QObject *eventReceiver, QObject *atbSystem, QObject *hmiConfig, QSettings const &settings) override;
    virtual PLUGIN_STATE initPlugin(QObject *eventReceiver, QSettings const &settings) override;
    virtual PLUGIN_STATE initVMCPlugin(QObject *eventReceiver, QObject *atbSystem, QObject *hmiConfig, const QSettings & settings) override;
    virtual PLUGIN_STATE initDCPlugin(QObject *eventReceiver, const QSettings & settings) override;

    // TASKS: Cash handling -------------------------------------------------------
    void requestStartCashInput(const QString & amount);
    void requestStopCashInput();
    void cashCollect();
    void cashAbort();

    // TASKS: printing ------------------------------------------------------------
    void requestPrintTicket(const QHash<QString, QVariant> & printingData);
    void requestPrintTicket(nsDeviceControllerInterface::TICKET_VARIANT ticketVariant, const QHash<QString, QVariant> & printingData);
    void requestPrintReceipt(const QHash<QString, QVariant> & printingData);
    void requestPrintReceipt(const QString & printingString);

    // TASKS: Account -------------------------------------------------------------
    void requestAccount();

    // mandantory ATBAPP plugin methods: ------------------------------------------
    nsDeviceControllerInterface::PLUGIN_STATE getState() override;
    QString const &getLastError() override;
    QString const &getLastErrorDescription() override;

    virtual QStringList getPluginInfoList() override;
    virtual const QString &getPluginInfo() override;

    // helpers e.g. for debug / log
    virtual QString const &getString(RESULT_STATE resultState) override;

public slots:
    virtual void onChangedProgramModeToSELL() override;
    virtual void onChangedProgramModeToSERVICE() override;
    virtual void onChangedProgramModeToIDLE() override;
    virtual void onChangedProgramModeToOOO() override;

    virtual void startPhysicalLayer() override;
    virtual void stopPhysicalLayer() override;
    virtual void reboot() override;
    virtual void reset() override;



signals:
    void printTicketFinished(RESULT_STATE resultState,
                             const QString & errorCode,
                             const QString & errorDescription);
    void printReceiptFinished(RESULT_STATE resultState,
                              const QString & errorCode,
                              const QString & errorDescription);

    /**
     * emitted on e.g. a coin input
     */
    void cashInputEvent(RESULT_STATE resultState,
                                CASH_STATE cashState,
                                const QString & newCashValue,
                                /* additional variables? */
                                const QString & errorCode,
                                const QString & errorDescription);

    /**
     * emitted if cashInput has been stopped, e.g. in result to task requestStopCashInput():
     *  -> shutter is blocked
     *  -> no cash input is possible
     *  -> coins are in cache
     */
    void cashInputFinished(RESULT_STATE resultState,
                                     const QString & newCashValue,
                                     /*  additional variables? */
                                     const QString & errorCode,
                                     const QString & errorDescription);


    /**
     * emitted e.g. if service door is opened
     */
    void requestModeSERVICE();

    /**
     * emitted e.g. if doors are closed
     */
    void requestModeIDLE();

    /**
     * emitted e.g. on severe errors
     */
    void requestModeOOO();

    /**
     * emitted e.g. if service door is opened
     */
    void requestAccountResponse(const QHash<QString, QVariant> & accountData);

    /**
     * emitted on error
     * depending on errorCode:
     * -> interrupt selling process
     * -> machine can go to state OOO
     * -> send error event to ISMAS
     * -> ...
     */
    void Error(
            /*  additional variables? */
            const QString & errorCode,
            const QString & errorDescription);


private:
   QString errorCode;
   QString errorDescription;
   QString pluginInfo;
   QStringList pluginInfoList;

   QList<int> templateList;

   QString serialPortName;

   bool useDebug;

   PLUGIN_STATE pluginState;

   QObject* eventReceiver;

   DBusControllerInterface* dbus;

   hwinf* hw;

   DeviceControllerDiag* diag;


   QTextCodec *codec;

   bool private_loadCashAgentLib(QString pluginName);

   quint8 currentSelectedTicketType;

   nsDeviceControllerInterface::CASH_STATE currentCashState;

   // counts failed hw->log_chkIfVaultRecordAvailable()
   int accountCheckCounter;

   // dbus
   int init_sc_dbus();

private slots:
   // printer

   void onPrinterDataPrepared();
   void onPrinterDataPreparedForTemplates();
   void onPrinterPrintNextTemplate();
   void onPrinterWaitForPrintingTicket();
   void onPrinterWaitForPrintingReceipt();

   void onPrintFinishedOK();
   void onPrintFinishedERR();

   // cash payment
   void onCashGotCoin();
   void onCashPayStopedSuccess();
   void onCashPayStopByMax();
   void onCashPayStopByEscrow();
   void onCashPayStopByError();
   void onCashPayStopByTimeout();

   // doors and hardware contacts
   void onServiceDoorOpened();
   void onVaultDoorOpened();
   void onCoinBoxRemoved();
   void onCoinBoxInserted();
   void onAllDoorsClosed();
   void onCBinAndAllDoorsClosed();

   // account handling
   void private_startAccount();
   void private_checkAccountData();
   void private_getAccountData();

   // measurement values
   void onNewVoltage(uint32_t voltage);

   void onWokeUp(uchar source);
};

#endif // ATBDEVICECONTROLLERPLUGIN_H
