#ifndef DEVICECONTROLLERINTERFACE_H
#define DEVICECONTROLLERINTERFACE_H

#include <QtPlugin>

#include <QSettings>
#include <QString>

#include "ATBAPPplugin.h"
#include "UnifiedDCVMCInterface.h"

namespace nsDeviceControllerInterface {
    using PLUGIN_STATE = ATBAPPplugin::PLUGIN_STATE;
    using RESULT_STATE = ATBAPPplugin::RESULT_STATE;
    using CASH_STATE = ATBAPPplugin::CASH_STATE;
    using TICKET_VARIANT = ATBAPPplugin::TICKET_VARIANT;
}

class DeviceControllerInterface : public QObject, public UnifiedDCVMCInterface
{
    Q_OBJECT
    Q_INTERFACES(ATBAPPplugin)
    Q_INTERFACES(UnifiedDCVMCInterface)

public:

    virtual ~DeviceControllerInterface() {}

    /**
     * @brief initDCPlugin
     * @param eventReceiver - QObject to receive ATBMachineEvents or HealthEvents
     * @param settings
     * @return
     */
    virtual PLUGIN_STATE initDCPlugin(QObject *eventReceiver, const QSettings & settings) = 0;

    // TASKS: Cash handling -------------------------------------------------------
    /**
     * enables coin input
     * amount = "0": pay-up
     * amount > "0": pay-down
     */
    virtual void requestStartCashInput(const QString & amount) = 0;

    /**
     * called e.g. on Button "NEXT" in pay-up (direct coin input)
     */
    virtual void requestStopCashInput() = 0;

    /**
     * called e.g. on Button "NEXT" in pay-up (direct coin input)
     */
    virtual void cashCollect() = 0;
    virtual void cashAbort() = 0;

    // TASKS: Account -------------------------------------------------------------

    virtual void requestAccount() = 0;


    // TASKS: printing ------------------------------------------------------------
    virtual void requestPrintTicket(const QHash<QString, QVariant> & printingData) = 0;
    virtual void requestPrintTicket(TICKET_VARIANT ticketVariant, const QHash<QString, QVariant> & printingData) = 0;
    virtual void requestPrintReceipt(const QHash<QString, QVariant> & printingData) = 0;
    virtual void requestPrintReceipt(const QString & printingString) = 0;


    // mandantory ATBAPP plugin methods:
    virtual nsDeviceControllerInterface::PLUGIN_STATE getState() = 0;
    virtual const QString & getLastError() = 0;
    virtual const QString & getLastErrorDescription() = 0;

    // return a plugin description in JSON or XML
    // -> ATBAPPplugin::getPluginInfo()

    // helpers e.g. for debug / log
    virtual const QString &getString(nsDeviceControllerInterface::RESULT_STATE resultState) = 0;

public slots:
    virtual void onChangedProgramModeToSELL() = 0;
    virtual void onChangedProgramModeToSERVICE() = 0;
    virtual void onChangedProgramModeToIDLE() = 0;
    virtual void onChangedProgramModeToOOO() = 0;

    virtual void startPhysicalLayer() = 0;
    virtual void stopPhysicalLayer() = 0;
    virtual void reboot() = 0;
    virtual void reset() = 0;

signals:
    void printTicketFinished(nsDeviceControllerInterface::RESULT_STATE resultState,
                                     const QString & errorCode,
                                     const QString & errorDescription);
    void printReceiptFinished(nsDeviceControllerInterface::RESULT_STATE resultState,
                                     const QString & errorCode,
                                     const QString & errorDescription);

    /**
     * emitted on e.g. a coin input
     */
    void cashInputEvent(nsDeviceControllerInterface::RESULT_STATE resultState,
                                nsDeviceControllerInterface::CASH_STATE cashState,
                                const QString & newCashValue,
                                /* additional variables? */
                                const QString & errorCode,
                                const QString & errorDescription);

    /**
     * emitted if cashInput has been stopped, e.g. in result to task requestStopCashInput():
     *  -> shutter is blocked
     *  -> no cash input is possible
     *  -> coins are in cache
     */
    void cashInputFinished(nsDeviceControllerInterface::RESULT_STATE resultState,
                                     const QString & newCashValue,
                                     /*  additional variables? */
                                     const QString & errorCode,
                                     const QString & errorDescription);

    /**
     * emitted e.g. if service door is opened
     */
    void requestModeSERVICE();

    /**
     * emitted e.g. if doors are closed
     */
    void requestModeIDLE();

    /**
     * emitted e.g. on severe errors
     */
    void requestModeOOO();

    /**
     * emitted e.g. if service door is opened
     */
    void requestAccountResponse(const QHash<QString, QVariant> & accountData);


    /**
     * emitted on error
     * depending on errorCode:
     * -> interrupt selling process
     * -> machine can go to state OOO
     * -> send error event to ISMAS
     * -> ...
     */
    void Error(
            /*  additional variables? */
            const QString & errorCode,
            const QString & errorDescription);

};

Q_DECLARE_INTERFACE(DeviceControllerInterface,
                    "eu.atb.ptu.plugin.DeviceControllerInterface/1.0")

#endif // DEVICECONTROLLERINTERFACE_H
